<?php

/**

 FOAFgen
 batch converter for LDIF addressbooks to FOAF profiles
 
 Version history:
    v0.3.0
        added vCard import support (plaintext format)
        now accepts multiple entries for each field value (eg. nickname or email addresses), shown as list box
        new features related to del.icio.us:
            seeAlso to RSS feed for interest links
            onlineAccount description
            seeAlso to foafalicious (http://philwilson.org:8180/foaflicious/)
    v0.2.3
        del.icio.us tag retrieval now also works for selected friends
    v0.2.2
        autodiscovery of foaf:interest's from del.icio.us user page
    v0.2.1
        added creation date and generatorAgent info
    v0.2.0
        complete rewrite of PHP code
        optional output of FOAF as file
        uses JS DOM to add extra information for each contact
        attempt autodiscovery of existing foaf profiles of selected friends using swordfish.rdfweb.org DB
        additional manual input of contacts not included in LDIF file
    v0.1.2
        added "foaf:interest" fields in personal info
        added IM chat ID field
        added "foaf:PersonalProfileDocument" node
        friends now described using "foaf:Person" types
        support for "rdfs:seeAlso" fields for friend's profiles

    v0.1.1
        initial public version
        support for LDIF URL's, not just local files
        "rel:worksWith" is pre-selected as relationship for entries with the same domain name in e-mail address as work homepage
        "foaf:nick" is only used if different to "foaf:name" value
        autostrip quotes from names

 @author Karsten Schmidt
 @version 0.3
 
*/

if (!isset($_POST['submit'])) header("Location: ./index.php");

/*
define('foafgenURL','http://localhost/eclipse/foafgen/');
define('RDFAPI_INCLUDE_DIR', 'c:/apache/xampp/htdocs/includes/rdfapi-php/api/');
// */

//*
define('foafgenURL','http://toxi.co.uk/foafgen/');
define('RDFAPI_INCLUDE_DIR', '/home/www/toxi/foafgen/rdfapi-php/api/');
// */

// include the RAP RDF engine
// more info: http://sourceforge.net/projects/rdfapi-php/
require_once(RDFAPI_INCLUDE_DIR.'RdfAPI.php');
require_once(RDFAPI_INCLUDE_DIR.PACKAGE_SYNTAX_RDF);
require_once(RDFAPI_INCLUDE_DIR.'vocabulary/FOAF_C.php');
require_once(RDFAPI_INCLUDE_DIR.'vocabulary/RSS_C.php');
require_once(RDFAPI_INCLUDE_DIR.'vocabulary/RDF_C.php');
require_once(RDFAPI_INCLUDE_DIR.'vocabulary/RDFS_C.php');

define(FOAFALICIOUS,'http://philwilson.org:8180/foaflicious/foaflicious.groovy?user=');

function generateFOAF() {
    include('relations.php');
    include('namespaces.php');
    
	// setup common RDF resources
	$model = ModelFactory :: getDefaultModel();
	$rdfTypeRes = new Resource(RDF_NAMESPACE_URI.'type');
	$foafPersonRes = new Resource($ns['foaf'].'Person');
	$foafNameRes = new Resource($ns['foaf'].'name');
	$foafNickRes = new Resource($ns['foaf'].'nick');
	$foafMboxRes = new Resource($ns['foaf'].'mbox');
    $foafHomepageRes = new Resource($ns['foaf'].'homepage');
    $rdfsSeeAlso = new Resource(RDF_SCHEMA_URI . 'seeAlso');
	$foafMboxRes_sha1 = new Resource($ns['foaf'].'mbox_sha1sum');
	$foafInterestRes = new Resource($ns['foaf'].'interest');
    $foafMakerRes = new Resource($ns['foaf'].'maker');
    $rssChannelRes = new Resource($ns['rss'].'channel');
    
    $useDelicious = isset($_POST['usedelicious']);
    
    // cycle through all entries
	$i = 1;
	$used = 0;
	$maxIndex = $_POST['numEntries'];
	$friends = array ();
	while ($i < $maxIndex) {
		if (isset ($_POST['a'.$i]) && $_POST['e'.$i] > '') {
			$friendNode = new BlankNode("friend".$i);
			$model->add(new Statement($friendNode, $rdfTypeRes, $foafPersonRes));
            
			$pname = trim($_POST['c'.$i]);
			if ($_POST['d'.$i] > '' && $_POST['d'.$i] != $pname)
				$model->add(new Statement($friendNode, $foafNickRes, new Literal($_POST['d'.$i])));
			if ($pname > '')
				$model->add(new Statement($friendNode, $foafNameRes, new Literal($pname)));
			if (!isset ($_POST['usesha1']))
				$model->add(new Statement($friendNode, $foafMboxRes, new Resource('mailto:'.$_POST["e".$i])));
            // always add mbox_sha1sum
            $sha1=sha1('mailto:'.$_POST["e".$i]);
            $model->add(new Statement($friendNode, $foafMboxRes_sha1, new Literal($sha1)));
            
            // homepage
            if (strlen($_POST['f'.$i])>7) {
                $model->add(new Statement($friendNode, $foafHomepageRes, new Resource($_POST["f".$i])));
            }
            // filled in seeAlso
            if (strlen($_POST['g'.$i])>7) {
                $model->add(new Statement($friendNode, $rdfsSeeAlso, new Resource($_POST["g".$i])));
            }
            
            // auto lookup for external seeAlso
            if (isset($_POST['autodiscover']) && !isset($errmsg)) {
                include_once('seealso_query.php');
                $query=new SeeAlsoQuery($sha1);
                $results=$query->execute();
                if ($results!=null) {
                    foreach($results as $seealso) {
                        $model->add(new Statement($friendNode, $rdfsSeeAlso, new Resource($seealso)));
                    }
                } else {
                    $errmsg=$query->getError()."\n";
                }
            }
            
            // get del.icio.us tags as foaf:interest
            if (strlen($_POST['h'.$i])>0) {
                if ($useDelicious) {
                    include_once('DeliciousScraper.class.php');
                    $scraper=new DeliciousScraper();
                    $tags=$scraper->getTagsForUser($_POST['h'.$i],10);
                    // add the tags discovered
                    if (count($tags)>0) {
                        foreach($tags as $tag) {
                            $model->add(new Statement($friendNode, $foafInterestRes, new Resource("http://del.icio.us/tag/$tag")));
                        }
                        sleep(2); // always wait after a successful request
                        set_time_limit(30); // reset time limit
                    } else $useDelicious=false; // disable future lookups if we've been kicked
                }
                $delRSS=new Resource('http://del.icio.us/rss/'.$_POST['h'.$i]);
                $delAccount=new Resource('http://del.icio.us/'.$_POST['h'.$i]);
                $model->add(new Statement($friendNode, RDFS::SEEALSO(), $delRSS));
                $model->add(new Statement($friendNode, RDFS::SEEALSO(), new Resource(FOAFALICIOUS.$_POST['h'.$i])));
                $model->add(new Statement($friendNode, FOAF::HOLDS_ACCOUNT(), $delAccount));
                addRSSFeedDesc($model,$friendNode,$delRSS);
                addOnlineAccountDesc($model,$delAccount,new Resource('http://del.icio.us'),$_POST['h'.$i]);
            }
            
			// get relationship type
			$frel = explode(":", $relOptions[$_POST['b'.$i]]);
			$frelNS = $ns[strtolower($frel[0])];
			// store definition in array
			array_push($friends, array ('def' => $friendNode, 'rel' => new Resource($frelNS.$frel[1])));
			$used ++;
		}
		$i ++;
	}

	// create "me" node
	$me = new BlankNode("me");
	$model->add(new Statement($me, $rdfTypeRes, $foafPersonRes));
    // check for del.icio.us user name
    if (strlen($_POST['me_delicious'])>0) {
        if ($useDelicious) {
            require_once('DeliciousScraper.class.php');
            $scraper=new DeliciousScraper();
            $tags=$scraper->getTagsForUser($_POST['me_delicious'],10);
            // check if other interest fields have been filled in
            $i=1;
            while(strlen($_POST["me_interest$i"])>0 && $i<6) $i++;
            // now add the tags discovered
            foreach($tags as $tag) {
                $_POST["me_interest$i"]="http://del.icio.us/tag/$tag";
                $i++;
            }
        }
    }
	// add personal data
	$needsRes = array ('depiction' => 1, 'workplaceHomepage' => 1, 'workInfoHomepage' => 1, 'schoolHomepage' => 1, 'homepage' => 1);
	foreach ($_POST as $key => $value) {
		if (substr($key, 0, 3) == 'me_') {
			if (strlen($value) > 0) {
				$key = substr($key, 3);
				if ($key == 'mbox') {
					if (isset ($_POST['usesha1']))
						$model->add(new Statement($me, $foafMboxRes_sha1, new Literal(sha1('mailto:'.$value))));
					else
						$model->add(new Statement($me, $foafMboxRes, new Resource('mailto:'.$value)));
				}
				elseif (substr($key,0,8)=='interest') {
					$key=substr($key,0,8);
                    // only use RDF resources if specified as URI
					if (substr($value,0,7)=="http://") {
						$model->add(new Statement($me, new Resource($ns['foaf'].$key), new Resource($value)));
					} else {
						$model->add(new Statement($me, new Resource($ns['foaf'].$key), new Literal($value)));
					}
				}
				elseif ($key=='imid') {
					if (strlen($_POST['imtype'])>0) {
						$model->add(new Statement($me, new Resource($ns['foaf'].$_POST['imtype']), new Literal($value)));
					}
				}
                elseif ($key=='delicious') {
                }
                elseif ($key=='phone') {
                    if (substr($value,0,4)!='tel:') $value='tel:'.$value;
                    $model->add(new Statement($me, new Resource($ns['foaf'].$key), new Resource($value)));
                }
				elseif ($needsRes[$key] == 1) {
					$model->add(new Statement($me, new Resource($ns['foaf'].$key), new Resource($value)));
				} else {
					$model->add(new Statement($me, new Resource($ns['foaf'].$key), new Literal($value)));
				}
			}
		}
	}
    
	// finally add all selected friends to "me" node
	foreach ($friends as $friend) {
		$model->add(new Statement($me, $friend['rel'], $friend['def']));
	}
	
    if (strlen($_POST['me_delicious'])>0) {
        $delRSS=new Resource('http://del.icio.us/rss/'.$_POST['me_delicious']);
        $delAccount=new Resource('http://del.icio.us/'.$_POST['me_delicious']);
        $model->add(new Statement($me, RDFS::SEEALSO(), $delRSS));
        $model->add(new Statement($me, RDFS::SEEALSO(), new Resource(FOAFALICIOUS.$_POST['me_delicious'])));
        $model->add(new Statement($me, FOAF::HOLDS_ACCOUNT(), $delAccount));
        addRSSFeedDesc($model,$me,$delRSS);
        addOnlineAccountDesc($model,$delAccount,new Resource('http://del.icio.us'),$_POST['me_delicious']);
    }
    
	// create profile description node with primary topic
	$profDoc = new Resource("");
	$model->add(new Statement($profDoc, $rdfTypeRes, new Resource($ns['foaf']."PersonalProfileDocument")));
	$model->add(new Statement($profDoc, $foafMakerRes, $me));
	$model->add(new Statement($profDoc, new Resource($ns['foaf']."primaryTopic"), $me));
	$model->add(new Statement($profDoc, new Resource($ns['admin']."generatorAgent"), new Resource(foafgenURL)));
    // create ISO 8601 date (PHP4 compatible)
    $stamp=date("Y-m-d").'T'.date("H:i:s").substr(date("T"),0,3);
    $dccreated= new Resource($ns['dcterms'].'created');
    $model->add(new Statement($profDoc, $dccreated, new Literal($stamp)));
    
    // generate RDF output using RAP's serializer
	$ser = new RdfSerializer();
    // add all used namespaces
    foreach($ns as $namespace => $nsURI) $ser->addNameSpacePrefix($namespace, $nsURI);

	$rdf = $ser->serialize($model);
	$model->close();
    return array('rdf' => $rdf, 'used' => $used, 'error' => $errmsg);
}

function addRSSFeedDesc(&$model,&$agent,&$rssRes) {
    $model->add(new Statement($rssRes, RDF::TYPE(), RSS::CHANNEL()));
    $model->add(new Statement($rssRes, FOAF::MAKER(), $agent));
}

function addOnlineAccountDesc(&$model,&$accountHome,&$servHome,$accountName) {
    $model->add(new Statement($accountHome, RDF::TYPE(), FOAF::ONLINE_ACCOUNT()));
    $model->add(new Statement($accountHome, FOAF::ACCOUNT_SERVICE_HOMEPAGE(), $servHome));
    $model->add(new Statement($accountHome, FOAF::ACCOUNT_NAME(), new Literal($accountName)));
}

$foaf=generateFOAF();

// attempt to write results to file
// if file has been written successfully, HTTP header will include a refresh tag to it

if ($foaf['used']>0) {
    if (isset($_POST['rdffile'])) {
        $genRDFfile='tmp/'.md5($foaf['rdf']).'.rdf';
        $out=@fopen($genRDFfile,'w');
        if ($out!=null && fwrite($out,$foaf['rdf'])) {
            fclose($out);
            $genRDFfile=foafgenURL.$genRDFfile;
            $fileInfo="<div>The above data has been saved in a temporary file. The download of which should start in a few seconds. If it doesn't, click on this link to <a href=\"redir.php?foaf=$genRDFfile\">download generated file</a>.<br /><em>Note:</em> This file will be deleted in max. 24 hours.</div>";
            $fileInfo.="<div><a href=\"http://www.w3.org/RDF/Validator/ARPServlet?URI=$genRDFfile\">Validate RDF</a></div>";
            $fileInfo.="<div><a href=\"http://xml.mfd-consult.dk/foaf/explorer/?foaf=$genRDFfile\">View in FOAFExplorer</a></div>";
        } else {
            $fileInfo="There was a server error when trying to write the file: \"$genRDFfile\". Please contact <a href=\"javascript:mailtoscramble('info','toxi.co.uk','?subject=FOAFgen');\">the author</a>.";
            unset($genRDFfile);
        }
    }
    /*
    if (!isset($_POST['rdffile'])) {
        unset($genRDFfile,$fileInfo);
    } // */
}

include('header.inc');

if ($foaf['used']>0) {
    echo "FOAF profile created with ".$foaf['used']." entries:<br/><br/>";
    echo '<textarea cols="100" rows="40" name="RDF">'.$foaf['rdf'].'</textarea>';
    if (isset($fileInfo)) echo "<div class=\"subsect\"><h3>Generated FOAF file output</h3>$fileInfo</div>";
    
    if (isset($foaf['error'])) {
        echo "<div class=\"subsect\"><h3>Autodiscovery of existing FOAF files</h3><div class=\"warning\">";
        if (strlen($foaf['error'])>80) $foaf['error']=substr($foaf['error'],0,80).'...';
        echo $foaf['error']."</div></div>\n";
    }
    echo '<div class="subsect"><h3>To do:</h3>Information how to properly publish the generated profile. For now, please see the related links on the right</div>';
}

echo '<div class="small">last updated: ' . date("F d Y H:i:s.", filemtime($_SERVER["SCRIPT_FILENAME"])) . '</div>';
include('footer.inc');
?>

