<?php
/**
* LDIF file parser to FOAF converter class
* conversion module for FOAFgen
*
* @author Karsten Schmidt info@toxi.co.uk
*/

class Ldif2FoafGen {

    var $entries;
    var $ldif2foaf = array ('cn' => 'name', 'givenName' => 'firstname', 'sn' => 'surname', 'mail' => 'mbox');
    
    /**
    * tokenize LDIF DN entry arguments and
    * convert attribute names into FOAF vocabulary
    * if they're set in the @see $ldif2foaf conversion table
    * all other attributes are ignored and will not be returned
    *
    * @return array with FOAF attribute-value pairs
    */
    function splitDN($dn) {
        $temp=explode(',',$dn);
        $result=array();
        foreach($temp as $token) {
            $tp=explode('=',$token);
            if (isset($this->ldif2foaf[$tp[0]])) $result[$this->ldif2foaf[$tp[0]]]=$this->trimAndStripQuotes($tp[1]);
        }
        return $result;
    }
    
    function splitToken($token) {
        $parts=explode(':',$token);
        return array('type' => trim($parts[0]), 'value' => $this->trimAndStripQuotes($parts[1]));
    }

    function trimAndStripQuotes($s) {
        return preg_replace("/\"|\[|\]/", "", trim($s));
    }
    
    function parseFile(&$ldif,$maxlimit=500) {
        $this->entries = array();
        $entryOpen = false;
        $entryData = null;
        $entryID = 0;
        while (!feof($ldif) && $entryID < $maxlimit) {
            $token = $this->splitToken(fgets($ldif, 256));
            $type = $token['type'];
            // start of new entry?
            if ($type == 'dn') {
                if (isset ($entryData)) {
                    // only use entries with set mbox & name values
                    if (strlen($entryData['mbox'])>0 && strlen($entryData['name'])>0) {
                        array_push($this->entries,$entryData);
                        $entryID ++;
                    }
                }
                $entryOpen = true;
                $entryData = $this->splitDN($token['value']);
            } else
            if ($entryOpen) {
                if (isset($this->ldif2foaf[$type])) {
                    // add more details to record
                    // if they match the allowed types in ldif2foaf table
                    if ($entryData[$this->ldif2foaf[$type]]==NULL) $entryData[$this->ldif2foaf[$type]] = $this->trimAndStripQuotes($token['value']);
                } else
                if (strpos($type,'nick')!=FALSE) {
                    $entryData['nick'] = $this->trimAndStripQuotes($token['value']);
                }
            }
        }
        // add final entry
        if (isset ($entryData)) {
            // only use entries with set mbox values
            if ($entryData['mbox'] != '') {
                array_push($this->entries,$entryData);
            }
        }
        fclose($ldif);
        return $this->entries;
    }
    
    function getEntryIterator() {
        require_once('SimpleIterator.php');
        return new SimpleIterator($this->entries);
    }
}
?>
