<?php
/**
* LDIF file parser to FOAF converter class
* conversion module for FOAFgen
*
* @author Karsten Schmidt info@toxi.co.uk
*/

class VCard2FoafGen {

    var $entries;
    var $vcard2foaf = array ('n' => 'name', 'email' => 'mbox', 'url' => 'homepage');

    function splitToken($token) {
        $parts=explode(':',strtolower($token));
        $attr=explode(';',trim(array_shift($parts)));
        $type=array_shift($attr);
        $values=explode(';',$this->trimAndStripQuotes(implode(':',$parts)));
        return array('type' => $type, 'attr' => $attr, 'value' => $values);
    }
    
    function trimAndStripQuotes($s) {
        return preg_replace('/"|\[|\]/', '', trim($s));
    }
    
    function parseFile(&$file,$maxlimit=500) {
        $this->entries = array();
        $entryOpen = false;
        $entryData = null;
        $entryID = 0;
        while (!feof($file) && $entryID < $maxlimit) {
            $token = $this->splitToken(fgets($file, 256));
            $type = $token['type'];
            // start of new entry?
            if ($type == 'begin') {
                if (isset ($entryData)) {
                    // only use entries with set mbox & name values
                    if (strlen($entryData['mbox'])>0 && strlen($entryData['name'])>0) {
                        array_push($this->entries,$entryData);
                        $entryID ++;
                    }
                }
                $entryOpen = true;
                $entryData = array();
            } elseif ($entryOpen) {
                if (isset($this->vcard2foaf[$type])) {
                    if ($type=='n') {
                        $entryData[$this->vcard2foaf[$type]] = $token['value'][1].' '.$token['value'][0];
                    } else {
                        if (!isset($entryData[$this->vcard2foaf[$type]])) {
                            $entryData[$this->vcard2foaf[$type]] = $token['value'][0];
                        } else {
                            if (gettype($entryData[$this->vcard2foaf[$type]])=='array') {
                                array_push($entryData[$this->vcard2foaf[$type]],$token['value'][0]);
                            } else {
                                $entryData[$this->vcard2foaf[$type]] = array($entryData[$this->vcard2foaf[$type]],$token['value'][0]);
                            }
                        }
                    }
                }
            }
        }
        // add final entry
        if (isset ($entryData)) {
            // only use entries with set mbox values
            if (strlen($entryData['mbox'])>0 && strlen($entryData['name'])>0) {
                array_push($this->entries,$entryData);
            }
        }
        fclose($file);
        return $this->entries;
    }
    
    function getEntryIterator() {
        require_once('SimpleIterator.php');
        return new SimpleIterator($this->entries);
    }
}
/*
$file=@fopen('vCards.vcf','r');
$gen=new VCard2FoafGen();
//print_r($gen->parseFile($file));
$a=array("hi");
echo gettype($a);
*/
?>
